/*
 *  bkl_helper_fxns.c
 *
 *  Copyright 2009. Quantum Corporation. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write the Free Software Foundation,
 * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */


#ifndef __KERNEL__
#define __KERNEL__
#endif

#ifndef MODULE
#define MODULE
#endif

#include <linux/kernel.h>
#define __NO_VERSION__      /* This isn't "the" file 
                             * of the kernel module */
#include <linux/module.h>
#include <linux/version.h>

#include <linux/autoconf.h>
#include <linux/sched.h>
#include <linux/smp_lock.h>

#include <linux/fs.h>
#include "bklpreempt.h"

#define MAXWAITERS 10

spinlock_t bklbug_waker_lock = SPIN_LOCK_UNLOCKED;


struct {
	void *waker_task;
	void *wakee_task;
} bklbug_waiters[MAXWAITERS];

void
record_waiter(void)
{
	int i;
	unsigned long flags;

	spin_lock_irqsave(&bklbug_waker_lock, flags);
	for(i = 0; i < MAXWAITERS; i++) {
		if (bklbug_waiters[i].waker_task == NULL) {
			bklbug_waiters[i].waker_task = current;
			bklbug_waiters[i].wakee_task = NULL;
			break;
		}
	}
	spin_unlock_irqrestore(&bklbug_waker_lock, flags);
}

void *
remove_waiter(void)
{
	int i;
	void *wakee = NULL;
	unsigned long flags;

	spin_lock_irqsave(&bklbug_waker_lock, flags);
	for(i = 0; i < MAXWAITERS; i++) {
		if (bklbug_waiters[i].waker_task == current) {
			bklbug_waiters[i].waker_task = NULL;
			wakee = bklbug_waiters[i].wakee_task;
			bklbug_waiters[i].wakee_task = NULL;
			break;
		}
 	}
	spin_unlock_irqrestore(&bklbug_waker_lock, flags);

	return(wakee);
}

/* Wrapper around the default_wake_function() to get the task that was
   woken up */
int bklbug_wake_up(wait_queue_t *curr, unsigned mode, int sync, void *key)
{
	int i;
	int result;
	unsigned long flags;
	struct task_struct *task;

	result = default_wake_function(curr, mode, sync, key);

	if (result) {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,12)
	    task = curr->private;
#else
	    task = curr->task;
#endif

		/* success */
		spin_lock_irqsave(&bklbug_waker_lock, flags);
		for(i = 0; i < MAXWAITERS; i++) {
			if (bklbug_waiters[i].waker_task == current) {
				bklbug_waiters[i].wakee_task = task;
				break;
			}
		}
		spin_unlock_irqrestore(&bklbug_waker_lock, flags);
	}

	return result;
}
