#! /bin/bash

#set -x

SLEEPERS=2
SPINNERS=0

make

insmod bklpreempt/bklbug.ko

if [ ! -e  /dev/bklbug ]; then
    mknod /dev/bklbug c 250 0
fi

for thread in `seq 0 $((SLEEPERS - 1))`; do
    test_bklpreempt/test_bklpreempt START_SLEEPER &
done

sleep $(( $SLEEPERS * 2 ))

test_bklpreempt/test_bklpreempt DUMP_WAITQUEUE

echo "TEST"

dmesg | tail -20 | awk '
  BEGIN {running=1;} 
  /QUEUE/ { 
    if ($0 ~ /running|disk sleep|sleeping/) {
	running=1;
	printf "running: %s\n", $0;
    } else {
	running=0;
	printf "not running: %s\n", $0;
    }
  }
  END {
      if (running == 1) {
	  exit 1;
      } else {
	  exit 0;
      }
  }'

if [ $? -ne 0 ]; then
   echo "FAILED to stop all sleepers: see /var/log/messages for task dump"
   logger "FAILED to stop all sleepers:"
   test_bklpreempt/test_bklpreempt DUMP_WAITQUEUE
   logger "Waking up final sleeper manually"
   echo "Waking up final sleeper and removing the module"
   test_bklpreempt/test_bklpreempt WAKEUP_SLEEPER
   usleep 500
   echo "*** BKL BUG PRESENT ***"
   if ! rmmod bklbug; then
      echo "Unable to unload bklbug driver"
      exit 2;
   fi
   exit 1;
fi

sleep 1

echo

if ! rmmod bklbug; then
    echo "Unable to rmmod bklbug!"
    exit 3;
fi

echo "*** NO BKL BUG DETECTED ***"
exit 0;
